
// Rar_Extractor 0.2.1

#include "unrarlib.h"

#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <setjmp.h>

#include "unrar/Rar_Extractor.h"

// Copyright (C) 2005 by Shay Green. MIT license.

// Handling deeply nested out-of-memory without exceptions is
// kind of hacky, and the global jmp_buf makes it thread-UNsafe.
// If C++ exception use is acceptable, rar_out_of_memory() could
// throw an exception that is caught in the wrapper functions.

static jmp_buf jmp_env;

void rar_out_of_memory() // called by Rar_Extractor
{
	longjmp( jmp_env, 1 );
}

// log error message to stderr when debugging
inline const char* log_error( const char* str )
{
	#ifndef NDEBUG
		if ( str && str != end_of_rar )
			fprintf( stderr, "RAR Error: %s\n", str );
	#endif
	return str;
}

int urarlib_get( void* output, unsigned long* size, char* filename,
		char* path, char *password )
{
	assert( !password ); // password not supported
	
	*(void**) output = NULL;
	*size = 0;
	
	// catch failed construction of rar extractor
	if ( setjmp( jmp_env ) )
		return false;
	
	void* volatile buf = NULL;
	Rar_Extractor rar;
	Std_File_Reader file;
	if ( setjmp( jmp_env ) )
		goto error;
	
	if ( log_error( file.open( path ) ) )
		goto error;
	
	if ( log_error( rar.open( &file ) ) )
		goto error;
	
	while ( !log_error( rar.next() ) )
	{
		if ( rar.info().is_file && 0 == strcmp( rar.info().name, filename ) )
		{
			buf = malloc( rar.info().size );
			if ( !buf )
				goto error;
			
			Mem_Writer out( buf, rar.info().size );
			if ( log_error( rar.extract( out ) ) )
				goto error;
			
			// success
			*(void**) output = buf;
			*size = rar.info().size;
			return true;
		}
	}
	
	 // not found
	 
error:
	free( buf );
	return false;
}

int urarlib_list( char* path, ArchiveList_struct** list )
{
	int count = 0;
	ArchiveList_struct* volatile item = NULL;
	*list = NULL;
	
	// catch failed construction of rar extractor
	if ( setjmp( jmp_env ) )
		return 0;
	
	Rar_Extractor rar;
	Std_File_Reader file;
	if ( setjmp( jmp_env ) )
		goto error;
	
	if ( log_error( file.open( path ) ) )
		goto error;
	
	if ( log_error( rar.open( &file ) ) )
		goto error;
	
	// significantly faster for solid archives
	rar.scan_only();
	
	while ( true )
	{
		const char* result = rar.next();
		if ( result == end_of_rar )
			return count; // success
		
		if ( log_error( result ) )
			goto error;
		
		if ( rar.info().is_file )
		{
			item = (ArchiveList_struct*) calloc( sizeof (ArchiveList_struct), 1 );
			if ( !item )
				goto error;
			
			item->item.NameSize = strlen( rar.info().name );
			item->item.Name = (char*) malloc( item->item.NameSize + 1 );
			if ( !item->item.Name )
				goto error;
			
			strcpy( item->item.Name, rar.info().name );
			item->item.UnpSize = rar.info().size;
			
			// insert at end
			*list = item;
			item->next = NULL;
			list = &item->next;
			count++;
		}
	}
	
error:
	free( item );
	return 0;
}

void urarlib_freelist( ArchiveList_struct* node )
{
	while ( node )
	{
		ArchiveList_struct* next = node->next;
		free( node->item.Name );
		free( node );
		node = next;
	}
}

